import java.util.*;

public class OptimusPrimeB {

    static boolean isFirstMove = true;
    static int[][] D;
    static char[][] moves;
    static int[][] initBoard;
    static int[][] points;
    static int primePointMultiplier = 2;
    static int N;

    public static char move (int[][] board, int myX, int myY, int otherX, int otherY)
    {
	if (isFirstMove) {
	    initBoard = board;
	    N = board.length;
	    isFirstMove = false;
	    computePlan ();
	}

	if (moves[myX][myY] == 's') {
	    return 's';
	}
	else if (moves[myX][myY] == 'h') {
	    // Try to move left
	    int tempX = myX - 1;
	    if ( (tempX == otherX) && (myY == otherY) ) {
		// Occupied. So move down.
		return 'd';
	    }
	    else {
		return 'h';
	    }
	}
	else {
	    // Planned strategy is to move down.
	    // Try and move down.
	    int tempY = myY - 1;
	    if ( (myX == otherX) && (tempY == otherY) ) {
		// Occupied. So move left.
		return 'h';
	    }
	    else {
		return 'd';
	    }
	}
    }

    static void computePlan ()
    {
	// compute points
	computePoints ();

	// Make space for dynamic programming.
	D = new int [N][N];
	moves = new char [N][N];

	// Initialize first row (bottom most)
	D[0][0] = points[0][0];
	moves[0][0] = 's';
	for (int x=1; x<N; x++) {
	    // Choice between move left and stay.
	    int stayPoints = points[x][0];
	    int leftPoints = D[x-1][0] + points[x][0];
	    if (leftPoints > stayPoints) {
		D[x][0] = leftPoints;
		moves[x][0] = 'h';
	    }
	    else {
		D[x][0] = stayPoints;
		moves[x][0] = 's';
	    }
	}

	// Now do the first column.
	for (int y=1; y<N; y++) {
	    // Choice between move down and stay.
	    int stayPoints = points[0][y];
	    int downPoints = D[0][y-1] + points[0][y];
	    if (downPoints > stayPoints) {
		D[0][y] = downPoints;
		moves[0][y] = 'd';
	    }
	    else {
		D[0][y] = stayPoints;
		moves[0][y] = 's';
	    }
	}

	// Proceed in increasing order of y (going upwards).
	for (int y=1; y<N; y++) {
	    // Compute for row j starting from left.
	    for (int x=1; x<N; x++) {
		// Choice between move left, move down, and stay.
		int stayPoints = points[x][y];
		int leftPoints = D[x-1][y] + points[x][y];
		int downPoints = D[x][y-1] + points[x][y];
		if ((leftPoints >= stayPoints) && (leftPoints > downPoints)) {
		    D[x][y] = leftPoints;
		    moves[x][y] = 'h';
		}
		else if ((downPoints >= stayPoints) && (downPoints > leftPoints)) {
		    D[x][y] = downPoints;
		    moves[x][y] = 'd';
		}
		else { // Stay
		    D[x][y] = stayPoints;
		    moves[x][y] = 's';
		}
	    }
	}

	printDown ("Points:", points);
	printDown ("D:", D);
    }

    static void computePoints ()
    {
	points = new int [N][N];
	for (int y=0; y<N; y++) {
	    for (int x=0; x<N; x++) {
		if (isPrime(initBoard[x][y])) {
		    points[x][y] = initBoard[x][y] * primePointMultiplier;
		}
		else {
		    points[x][y] = -initBoard[x][y];
		}
	    }
	}
    }

    static boolean isPrime (int n)
    {
	for (int i=2; i<=n/2; i++) {
	    if (n % i == 0) {
		return false;
	    }
	}
	return true;
    }

    static void printDown (String msg, int[][] A)
    {
	System.out.println (msg);
	for (int y=N-1; y>=0; y--) {
	    for (int x=0; x<N; x++) {
		System.out.printf ("%5d",A[x][y]);
	    }
	    System.out.println ();
	}
    }

}
