
import java.util.*;
import java.io.*;

public class PrimeGame {

    // Game variables and data:
    static int N = 10;
    static int[][] board = new int [N][N];
    static int[][] points = new int [N][N];
    static int primePointMultiplier = 10;

    // Board structure and indexing:
    // We'll use x,y coordinates (x going left to right, y from below to above).
    // So, board[x][y] would need to be printed from high-y downwards.

    // GUI stuff:
    static String negativeColor = "yellow";
    static String positiveColor = "green";
    static String playerAColor = "red";
    static String playerBColor = "blue";
    static double sleepTime = 0.5;
    
    // Player locations.
    static int playerAX=0, playerAY=N-1;
    static int playerBX=N-1, playerBY=N-1;

    // Scores:
    static int playerApoints=0, playerBpoints=0;

    // Default:
    static int gameType = 1;
    // Game types: 
    //   1: PlayerA vs PlayerB         -- Your PlayerA vs your PlayerB
    //   2: RandomA vs Manual          -- Manually play (as B) against Random
    //   3: PlayerA vs Manual          -- Manually play against your PlayerA
    //   4: PlayerA vs RandomB         -- Your PlayerA against Random
    //   5: RandomA vs RandomB         -- Just watch one Random vs another.
    //   6: PlayerA vs OptimusPrimeB   -- Your player against OptimusPrime
    //   7: Manual vs OptimusPrimeB    -- Play manually against OP

    public static void main (String[] argv)
    {
	if ((argv == null) || (argv.length != 1)) {
	    System.out.println ("Usage: java Prime <gametype>");
	    System.exit (0);
	}

	// Now argv[0] has the game type.
	gameType = stringToInt (argv[0]);
	if ((gameType < 1) || (gameType > 7)) {
	    System.out.println ("Illegal game type: " + gameType);
	    System.exit (0);
	}

	makeBoard ();
	drawBoard ();

	// Draw the players.
	updatePlayerLocation (playerAColor, playerAX, playerAY, playerAX, playerAY);
	updatePlayerLocation (playerBColor, playerBX, playerBY, playerBX, playerBY);
	executeGame ();
    }


    static void executeGame ()
    {
	boolean over = false;
	boolean canMoveA = true, canMoveB = true;
	int numSteps = 0;

	while (! over) {

	    numSteps++;

	    canMoveA = checkCanMoveA ();
	    if (canMoveA) {                  // A can move
		boolean isLegal = moveA ();  // A's move worked.
		if (! isLegal) {
		    playerApoints = -1;      // If not, A loses.
		    break;                   // End the game right away.
		}
		System.out.println ("Step " + numSteps + ": Player A moved to: x=" + playerAX + " y=" + playerAY);
	    }

	    canMoveB = checkCanMoveB ();     // Same for B.
	    if (canMoveB) {
		boolean isLegal = moveB ();
		if (! isLegal) {
		    playerBpoints = -1;
		    break;
		}
		System.out.println ("Step " + numSteps + ": Player B moved to: x=" + playerBX + " y=" + playerBY + "\n");
	    }

	    over = (! canMoveA) && (! canMoveB);
	    if (numSteps > 2*N-2) {
		over = true;
	    }

	} // endwhile

	// Print who won/lost, points etc.
	printEndStuff ();

    }

    static boolean checkCanMoveA ()
    {
	// You can't move A if it's on the lowest level and x >= N.
	return ( ! ((playerAY == 0) && (playerAX >= N-1)) );
    }

    static boolean checkCanMoveB ()
    {
	// You can't move B if it's on the lowest level and x <= 0
	return ( ! ((playerBY == 0) && (playerBX <= 0)) );
    }


    static boolean moveA ()
    {
	// First, get the move:
	char c = ' ';
	int[][] b = copyArray (board);
	if (gameType == 1) {
	    // PlayerA vs Player B
	    sleep (sleepTime);
	    c = PlayerA.move (b, playerAX, playerAY, playerBX, playerBY);
	}
	else if (gameType == 2) {
	    // PlayerA vs Manual
	    c = RandomA.move (b, playerAX, playerAY, playerBX, playerBY);
	}
	else if (gameType == 3) {
	    // RandomA vs Manual
	    c = PlayerA.move (b, playerAX, playerAY, playerBX, playerBY);
	}
	else if (gameType == 4) {
	    // PlayerA vs RandomB
	    sleep (sleepTime);
	    c = PlayerA.move (b, playerAX, playerAY, playerBX, playerBY);
        }
	else if (gameType == 5) {
	    // RandomA vs RandomB
	    sleep (sleepTime);
	    c = RandomA.move (b, playerAX, playerAY, playerBX, playerBY);
	}
	else if (gameType == 6) {
	    // PlayerA vs OptimusPrimeB
	    sleep (sleepTime);
	    c = PlayerA.move (b, playerAX, playerAY, playerBX, playerBY);
	}
	else if (gameType == 7) {
	    // Manual vs OptimusPrimeB
	    c = manualMove ();
	}
	else {
	    System.out.println ("ERROR: invalid game type: " + gameType);
	    System.exit (0);
	}

	System.out.println (" >> moveA: " + c);

	// Now evaluate legality.

	int tempX = playerAX, tempY = playerAY;

	if (c == 'h') {
	    tempX = playerAX + 1;
	    if ( (tempX >= N) || 
		 ( (tempX == playerBX) && (playerAY == playerBY) ) ) {
		// Illegal.
		System.out.println (" >>>> Illegal: x=" + tempX + " y=" + tempY);
		return false;
	    }
	}
	else if (c == 'd') {
	    tempY = playerAY - 1;
	    if ( (tempY < 0) ||
		 ( (playerAX == playerBX) && (tempY == playerBY) ) ) {
		// Illegal.
		System.out.println (" >>>> Illegal: x=" + tempX + " y=" + tempY);
		return false;
	    }
	}
	else if (c == 's') {
	    // Stay.
	}
	else {
	    // Illegal.
	    return false;
	}

	// Apply move and update score. First draw.
	updatePlayerLocation (playerAColor, playerAX, playerAY, tempX, tempY);

	playerAX = tempX;
	playerAY = tempY;

	playerApoints += points[tempX][tempY];
	points[tempX][tempY] = 0;

	return true;
    }

    static boolean moveB ()
    {
	// First, get the move:
	char c = ' ';
	int[][] b = copyArray (board);
	if (gameType == 1) {
	    // PlayerA vs Player B
	    sleep (sleepTime);
	    c = PlayerB.move (b, playerBX, playerBY, playerAX, playerAY);
	}
	else if (gameType == 2) {
	    // PlayerA vs Manual
	    //c = manualMoveB (b, playerBX, playerBY, playerAX, playerAY);
	    c = manualMove ();
	}
	else if (gameType == 3) {
	    // RandomA vs Manual
	    c = manualMove ();
	}
	else if (gameType == 4) {
	    // PlayerA vs RandomB
	    sleep (sleepTime);
	    c = RandomB.move (b, playerBX, playerBY, playerAX, playerAY);
        }
	else if (gameType == 5) {
	    // RandomA vs RandomB
	    sleep (sleepTime);
	    c = RandomB.move (b, playerBX, playerBY, playerAX, playerAY);
	}
	else if (gameType == 6) {
	    // PlayerA vs OptimusPrimeB
	    sleep (sleepTime);
	    c = OptimusPrimeB.move (b, playerBX, playerBY, playerAX, playerAY);
	}
	else if (gameType == 7) {
	    // Manual vs OptimusPrimeB
	    c = OptimusPrimeB.move (b, playerBX, playerBY, playerAX, playerAY);
	}
	else {
	    System.out.println ("ERROR: invalid game type: " + gameType);
	    System.exit (0);
	}

	System.out.println (" >> moveB: " + c);

	// Now evaluate legality.
	int tempX = playerBX, tempY = playerBY;
	if (c == 'h') {
	    tempX = playerBX - 1;
	    if ( (tempX < 0) ||
		 ( (tempX == playerAX) && (playerAY == playerBY) ) ) {
		// Illegal.
		System.out.println (" >>>> Illegal: x=" + tempX + " y=" + playerBY);
		return false;
	    }
	}
	else if (c == 'd') {
	    tempY = playerBY - 1;
	    if ( (tempY < 0) || 
		 ( (playerAX == playerBX) && (tempY == playerAY) ) ) {
		// Illegal.
		System.out.println (" >>>> Illegal: x=" + tempX + " y=" + tempY);
		return false;
	    }
	}
	else if (c == 's') {
	    // Stay.
	}
	else {
	    // Illegal.
	    return false;
	}

	// Apply move and update score.
	updatePlayerLocation (playerBColor, playerBX, playerBY, tempX, tempY);

	playerBX = tempX;
	playerBY = tempY;

	playerBpoints += points[tempX][tempY];
	points[tempX][tempY] = 0;

	return true;
    }

    static void printEndStuff ()
    {
	String winStr = "A wins!";
	if (playerApoints < playerBpoints) {
	    winStr = "B wins!";
	}
	else if (playerApoints == playerBpoints) {
	    winStr = "Draw!";
	}
	String outputStr = "A's score=" + playerApoints + " B's score=" + playerBpoints + "  Result: " + winStr;
	DrawTool.writeTopString (outputStr);
	System.out.println (outputStr);
    }


    static void makeBoard ()
    {
	// First make an array of numbers 1 .. M=N*N
	int[] numbers = GameArrayTool.shiftedArray (N);

	// NOTE: WE WILL COMMENT THIS PART OUT INITIALLY, so that
	// the game is predictable. Uncomment the line of code below
	// after you've had time to put your strategy together.
	// GameArrayTool.shuffle (numbers);

	// Next, place board.
	GameArrayTool.putNumbersOnBoard (board, numbers);

	System.out.println ("Initial board:");
	GameArrayTool.printBoard (board);

	// Now do points. 
	for (int y=0; y<N; y++) {
	    for (int x=0; x<N; x++) {
		if (GameArrayTool.isPrime(board[x][y])) {
		    points[x][y] = board[x][y] * primePointMultiplier;
		}
		else {
		    points[x][y] = -board[x][y];
		}
	    }
	}
    }

    static int[][] copyArray (int[][] A)
    {
	// Make a copy to prevent strategic tampering.
	int[][] B = new int [A.length][A[0].length];
	for (int i=0; i<N; i++) {
	    for (int j=0; j<N; j++) {
		B[i][j] = A[i][j];
	    }
	}	

	return B;
    }


    static void drawBoard ()
    {
	DrawTool.setXYRange (0,N, 0,N);
	DrawTool.display ();

	// Draw the circles and labels.
	for (int y=0; y<N; y++) {
	    for (int x=0; x<N; x++) {
		// Draw circle centered at x+0.5, y+0.5, r=0.5
		if (points[x][y] < 0) {
		    DrawTool.setCircleColor (negativeColor);
		}
		else {
		    DrawTool.setCircleColor (positiveColor);
		}
		DrawTool.drawCircle (x+0.5, y+0.5, 0.5);
		DrawTool.drawSmallLabel (x+0.4, y+0.4, ""+board[x][y]);
	    }
	}

    }



    static char manualMove ()
    {
	// Get string typed into textbox of DrawTool:
	String s = DrawTool.getInputString ();
	if (s.equalsIgnoreCase("d")) {
	    return 'd';
	}
	else if (s.equalsIgnoreCase("h")) {
	    return 'h';
	}
	else {
	    DrawTool.writeTopString ("Invalid input");
	    return ' ';
	}
    }

    static char manualMoveB (int[][] board, int myX, int myY, int otherX, int otherY)
    {
	// Get string typed into textbox of DrawTool:
	String s = DrawTool.getInputString ();
	if (s.equalsIgnoreCase("d")) {
	    return 'd';
	}
	else if (s.equalsIgnoreCase("h")) {
	    return 'h';
	}
	else {
	    DrawTool.writeTopString ("Invalid input");
	    return ' ';
	}
    }

    static void updatePlayerLocation (String color, int currentX, int currentY, int nextX, int nextY)
    {
	DrawTool.setCircleColor ("white");
	DrawTool.drawCircle (currentX+0.5, currentY+0.5, 0.4);
	DrawTool.setCircleColor (color);
	DrawTool.drawCircle (nextX+0.5, nextY+0.5, 0.4);
    }

    static void sleep (double sleepTimeSeconds)
    {
	try {
	    int timeMilliSeconds = (int) (1000 * sleepTimeSeconds);
	    Thread.sleep (timeMilliSeconds);
	}
	catch (Exception e) {
	}
    }

    static int stringToInt (String s)
    {
	try {
	    // Remove surrounding whitepace.
	    s = s.trim ();
	    // Parse.
	    int k = Integer.parseInt (s);
	    return k;
	}
	catch (Exception e) {
	    // If the string was illegal, our action is harsh: halt.
	    System.out.println ("Illegal string for integer: " + s);
	    System.exit (0);
	    // We won't be reaching here but the compiler needs a 
	    // return to be written.
	    return -1;
	}
    }

}
